@@ -3,7 +3,7 @@ package Moose::Error::Croak;
 use strict;
 use warnings;
 
-our $VERSION   = '0.82';
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -26,10 +26,13 @@ Moose::Error::Croak - Prefer C<croak>
 
 =head1 SYNOPSIS
 
-	use metaclass => (
-        metaclass => "Moose::Meta::Class",
-        error_class => "Moose::Error::Croak",
+    # Metaclass definition must come before Moose is used.
+    use metaclass (
+        metaclass => 'Moose::Meta::Class',
+        error_class => 'Moose::Error::Croak',
     );
+    use Moose;
+    # ...
 
 =head1 DESCRIPTION
 
