@@ -4,7 +4,7 @@ package Moose::Meta::Method::Accessor;
 use strict;
 use warnings;
 
-our $VERSION   = '0.63';
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -17,33 +17,36 @@ sub _error_thrower {
 }
 
 sub _eval_code {
-    my ( $self, $code ) = @_;
+    my ( $self, $source ) = @_;
 
     # NOTE:
     # set up the environment
-    my $attr        = $self->associated_attribute;
-    my $attr_name   = $attr->name;
-    my $meta        = $self,
-
-    my $type_constraint_obj  = $attr->type_constraint;
-    my $type_constraint_name = $type_constraint_obj && $type_constraint_obj->name;
-    my $type_constraint      = $type_constraint_obj
+    my $attr = $self->associated_attribute;
+    my $type_constraint_obj = $attr->type_constraint;
+    my $environment = {
+        '$attr' => \$attr,
+        '$meta' => \$self,
+        '$type_constraint_obj' => \$type_constraint_obj,
+        '$type_constraint' => \($type_constraint_obj
                                    ? $type_constraint_obj->_compiled_type_constraint
-                                   : undef;
+                                   : undef),
+    };
 
-    #warn "code for $attr_name =>\n" . $code . "\n";
-    my $sub = eval $code;
-    $self->throw_error("Could not create writer for '$attr_name' because $@ \n code: $code", error => $@, data => $code ) if $@;
-    return $sub;
+    #warn "code for " . $attr->name . " =>\n" . $source . "\n";
+    my ( $code, $e ) = $self->_compile_code( environment => $environment, code => $source );
 
+    $self->throw_error(
+        "Could not create writer for '${\$self->associated_attribute->name}' because $e \n code: $source",
+        error => $e, data => $source )
+        if $e;
+
+    return $code;
 }
 
-sub generate_accessor_method_inline {
+sub _generate_accessor_method_inline {
     my $self        = $_[0];
     my $attr        = $self->associated_attribute;
-    my $attr_name   = $attr->name;
     my $inv         = '$_[0]';
-    my $slot_access = $self->_inline_access($inv, $attr_name);
     my $value_name  = $self->_value_needs_copy ? '$val' : '$_[1]';
 
     $self->_eval_code('sub { ' . "\n"
@@ -51,10 +54,11 @@ sub generate_accessor_method_inline {
     . 'if (scalar(@_) >= 2) {' . "\n"
         . $self->_inline_copy_value . "\n"
         . $self->_inline_check_required . "\n"
-        . $self->_inline_check_coercion . "\n"
+        . $self->_inline_check_coercion($value_name) . "\n"
         . $self->_inline_check_constraint($value_name) . "\n"
+        . $self->_inline_get_old_value_for_trigger($inv, $value_name) . "\n"
         . $self->_inline_store($inv, $value_name) . "\n"
-        . $self->_inline_trigger($inv, $value_name) . "\n"
+        . $self->_inline_trigger($inv, $value_name, '@old') . "\n"
     . ' }' . "\n"
     . $self->_inline_check_lazy($inv) . "\n"
     . $self->_inline_post_body(@_) . "\n"
@@ -62,32 +66,31 @@ sub generate_accessor_method_inline {
     . ' }');
 }
 
-sub generate_writer_method_inline {
+sub _generate_writer_method_inline {
     my $self        = $_[0];
     my $attr        = $self->associated_attribute;
-    my $attr_name   = $attr->name;
     my $inv         = '$_[0]';
-    my $slot_access = $self->_inline_get($inv, $attr_name);
+    my $slot_access = $self->_inline_get($inv);
     my $value_name  = $self->_value_needs_copy ? '$val' : '$_[1]';
 
     $self->_eval_code('sub { '
     . $self->_inline_pre_body(@_)
     . $self->_inline_copy_value
     . $self->_inline_check_required
-    . $self->_inline_check_coercion
+    . $self->_inline_check_coercion($value_name)
     . $self->_inline_check_constraint($value_name)
+    . $self->_inline_get_old_value_for_trigger($inv, $value_name) . "\n"
     . $self->_inline_store($inv, $value_name)
     . $self->_inline_post_body(@_)
-    . $self->_inline_trigger($inv, $value_name)
+    . $self->_inline_trigger($inv, $value_name, '@old')
     . ' }');
 }
 
-sub generate_reader_method_inline {
+sub _generate_reader_method_inline {
     my $self        = $_[0];
     my $attr        = $self->associated_attribute;
-    my $attr_name   = $attr->name;
     my $inv         = '$_[0]';
-    my $slot_access = $self->_inline_get($inv, $attr_name);
+    my $slot_access = $self->_inline_get($inv);
 
     $self->_eval_code('sub {'
     . $self->_inline_pre_body(@_)
@@ -108,40 +111,73 @@ sub _value_needs_copy {
     return $attr->should_coerce;
 }
 
-sub generate_reader_method { shift->generate_reader_method_inline(@_) }
-sub generate_writer_method { shift->generate_writer_method_inline(@_) }
-sub generate_accessor_method { shift->generate_accessor_method_inline(@_) }
+sub _instance_is_inlinable {
+    my $self = shift;
+    return $self->associated_attribute->associated_class->instance_metaclass->is_inlinable;
+}
+
+sub _generate_reader_method {
+    my $self = shift;
+    $self->_instance_is_inlinable ? $self->_generate_reader_method_inline(@_)
+                                  : $self->SUPER::_generate_reader_method(@_);
+}
+
+sub _generate_writer_method {
+    my $self = shift;
+    $self->_instance_is_inlinable ? $self->_generate_writer_method_inline(@_)
+                                  : $self->SUPER::_generate_writer_method(@_);
+}
+
+sub _generate_accessor_method {
+    my $self = shift;
+    $self->_instance_is_inlinable ? $self->_generate_accessor_method_inline(@_)
+                                  : $self->SUPER::_generate_accessor_method(@_);
+}
+
+sub _generate_predicate_method {
+    my $self = shift;
+    $self->_instance_is_inlinable ? $self->_generate_predicate_method_inline(@_)
+                                  : $self->SUPER::_generate_predicate_method(@_);
+}
+
+sub _generate_clearer_method {
+    my $self = shift;
+    $self->_instance_is_inlinable ? $self->_generate_clearer_method_inline(@_)
+                                  : $self->SUPER::_generate_clearer_method(@_);
+}
 
 sub _inline_pre_body  { '' }
 sub _inline_post_body { '' }
 
 sub _inline_check_constraint {
     my ($self, $value) = @_;
-    
+
     my $attr = $self->associated_attribute;
-    my $attr_name = $attr->name;
-    
+
     return '' unless $attr->has_type_constraint;
-    
-    my $type_constraint_name = $attr->type_constraint->name;
+
+    my $attr_name = quotemeta( $attr->name );
 
     qq{\$type_constraint->($value) || } . $self->_inline_throw_error(qq{"Attribute ($attr_name) does not pass the type constraint because: " . \$type_constraint_obj->get_message($value)}, "data => $value") . ";";
 }
 
 sub _inline_check_coercion {
-    my $attr = (shift)->associated_attribute;
-    
-    return '' unless $attr->should_coerce;
-    return '$val = $attr->type_constraint->coerce($_[1]);'
+    my ($self, $value) = @_;
+
+    my $attr = $self->associated_attribute;
+
+    return '' unless $attr->should_coerce && $attr->type_constraint->has_coercion;
+    return "$value = \$attr->type_constraint->coerce($value);";
 }
 
 sub _inline_check_required {
     my $self = shift;
     my $attr = $self->associated_attribute;
 
-    my $attr_name = $attr->name;
-    
     return '' unless $attr->is_required;
+
+    my $attr_name = quotemeta( $attr->name );
+
     return qq{(\@_ >= 2) || } . $self->_inline_throw_error(qq{"Attribute ($attr_name) is required, so cannot be set to undef"}) . ';' # defined $_[1] is not good enough
 }
 
@@ -152,16 +188,14 @@ sub _inline_check_lazy {
 
     return '' unless $attr->is_lazy;
 
-    my $slot_access = $self->_inline_access($instance, $attr->name);
-
-    my $slot_exists = $self->_inline_has($instance, $attr->name);
+    my $slot_exists = $self->_inline_has($instance);
 
     my $code = 'unless (' . $slot_exists . ') {' . "\n";
     if ($attr->has_type_constraint) {
         if ($attr->has_default || $attr->has_builder) {
             if ($attr->has_default) {
                 $code .= '    my $default = $attr->default(' . $instance . ');'."\n";
-            } 
+            }
             elsif ($attr->has_builder) {
                 $code .= '    my $default;'."\n".
                          '    if(my $builder = '.$instance.'->can($attr->builder)){ '."\n".
@@ -169,30 +203,27 @@ sub _inline_check_lazy {
                          '        ' . $self->_inline_throw_error(q{sprintf "%s does not support builder method '%s' for attribute '%s'", ref(} . $instance . ') || '.$instance.', $attr->builder, $attr->name') .
                          ';'. "\n    }";
             }
-            $code .= '    $default = $type_constraint_obj->coerce($default);'."\n"  if $attr->should_coerce;
-            $code .= '    ($type_constraint->($default))' .
-                     '            || ' . $self->_inline_throw_error('"Attribute (" . $attr_name . ") does not pass the type constraint ("' .
-                     '           . $type_constraint_name . ") with " . (defined($default) ? overload::StrVal($default) : "undef")' ) . ';' 
-                     . "\n";
-            $code .= '    ' . $self->_inline_init_slot($attr, $instance, $slot_access, '$default') . "\n";
-        } 
+            $code .= $self->_inline_check_coercion('$default') . "\n";
+            $code .= $self->_inline_check_constraint('$default') . "\n";
+            $code .= '    ' . $self->_inline_init_slot($attr, $instance, '$default') . "\n";
+        }
         else {
-            $code .= '    ' . $self->_inline_init_slot($attr, $instance, $slot_access, 'undef') . "\n";
+            $code .= '    ' . $self->_inline_init_slot($attr, $instance, 'undef') . "\n";
         }
 
     } else {
         if ($attr->has_default) {
-            $code .= '    ' . $self->_inline_init_slot($attr, $instance, $slot_access, ('$attr->default(' . $instance . ')')) . "\n";            
-        } 
+            $code .= '    ' . $self->_inline_init_slot($attr, $instance, ('$attr->default(' . $instance . ')')) . "\n";
+        }
         elsif ($attr->has_builder) {
-            $code .= '    if (my $builder = '.$instance.'->can($attr->builder)) { ' . "\n" 
-                  .  '       ' . $self->_inline_init_slot($attr, $instance, $slot_access, ($instance . '->$builder'))           
+            $code .= '    if (my $builder = '.$instance.'->can($attr->builder)) { ' . "\n"
+                  .  '       ' . $self->_inline_init_slot($attr, $instance, ($instance . '->$builder'))
                   .  "\n    } else {\n"
                   .  '        ' . $self->_inline_throw_error(q{sprintf "%s does not support builder method '%s' for attribute '%s'", ref(} . $instance . ') || '.$instance.', $attr->builder, $attr->name')
                   .  ';'. "\n    }";
-        } 
+        }
         else {
-            $code .= '    ' . $self->_inline_init_slot($attr, $instance, $slot_access, 'undef') . "\n";
+            $code .= '    ' . $self->_inline_init_slot($attr, $instance, 'undef') . "\n";
         }
     }
     $code .= "}\n";
@@ -200,63 +231,74 @@ sub _inline_check_lazy {
 }
 
 sub _inline_init_slot {
-    my ($self, $attr, $inv, $slot_access, $value) = @_;
+    my ($self, $attr, $inv, $value) = @_;
     if ($attr->has_initializer) {
         return ('$attr->set_initial_value(' . $inv . ', ' . $value . ');');
     }
     else {
-        return ($slot_access . ' = ' . $value . ';');
-    }    
+        return $self->_inline_store($inv, $value);
+    }
 }
 
 sub _inline_store {
     my ($self, $instance, $value) = @_;
     my $attr = $self->associated_attribute;
-    
+
     my $mi = $attr->associated_class->get_meta_instance;
-    my $slot_name = sprintf "'%s'", $attr->slots;
-    
-    my $code = $mi->inline_set_slot_value($instance, $slot_name, $value)    . ";";
-    $code   .= $mi->inline_weaken_slot_value($instance, $slot_name, $value) . ";"
+
+    my $code = $mi->inline_set_slot_value($instance, $attr->slots, $value)    . ";";
+    $code   .= $mi->inline_weaken_slot_value($instance, $attr->slots, $value) . ";"
         if $attr->is_weak_ref;
     return $code;
 }
 
-sub _inline_trigger {
-    my ($self, $instance, $value) = @_;
+sub _inline_get_old_value_for_trigger {
+    my ( $self, $instance ) = @_;
+
     my $attr = $self->associated_attribute;
     return '' unless $attr->has_trigger;
-    return sprintf('$attr->trigger->(%s, %s, $attr);', $instance, $value);
+
+    my $mi = $attr->associated_class->get_meta_instance;
+    my $pred = $mi->inline_is_slot_initialized($instance, $attr->name);
+
+    return
+          'my @old = '
+        . $pred . q{ ? }
+        . $self->_inline_get($instance) . q{ : ()} . ";\n";
+}
+
+sub _inline_trigger {
+    my ($self, $instance, $value, $old_value) = @_;
+    my $attr = $self->associated_attribute;
+    return '' unless $attr->has_trigger;
+    return sprintf('$attr->trigger->(%s, %s, %s);', $instance, $value, $old_value);
 }
 
 sub _inline_get {
     my ($self, $instance) = @_;
     my $attr = $self->associated_attribute;
-    
-    my $mi = $attr->associated_class->get_meta_instance;
-    my $slot_name = sprintf "'%s'", $attr->slots;
 
-    return $mi->inline_get_slot_value($instance, $slot_name);
+    my $mi = $attr->associated_class->get_meta_instance;
+
+    return $mi->inline_get_slot_value($instance, $attr->slots);
 }
 
 sub _inline_access {
     my ($self, $instance) = @_;
     my $attr = $self->associated_attribute;
-    
-    my $mi = $attr->associated_class->get_meta_instance;
-    my $slot_name = sprintf "'%s'", $attr->slots;
 
-    return $mi->inline_slot_access($instance, $slot_name);
+    my $mi = $attr->associated_class->get_meta_instance;
+
+    return $mi->inline_slot_access($instance, $attr->slots);
 }
 
 sub _inline_has {
     my ($self, $instance) = @_;
     my $attr = $self->associated_attribute;
-    
-    my $mi = $attr->associated_class->get_meta_instance;
-    my $slot_name = sprintf "'%s'", $attr->slots;
 
-    return $mi->inline_is_slot_initialized($instance, $slot_name);
+    my $mi = $attr->associated_class->get_meta_instance;
+
+    return $mi->inline_is_slot_initialized($instance, $attr->slots);
 }
 
 sub _inline_auto_deref {
@@ -275,7 +317,9 @@ sub _inline_auto_deref {
         $sigil = '%';
     }
     else {
-        $self->throw_error("Can not auto de-reference the type constraint '" . $type_constraint->name . "'", type_constraint => $type_constraint );
+        $self->throw_error( "Can not auto de-reference the type constraint '"
+                . quotemeta( $type_constraint->name )
+                . "'", type_constraint => $type_constraint );
     }
 
     "(wantarray() ? $sigil\{ ( $ref_value ) || return } : ( $ref_value ) )";
@@ -293,37 +337,16 @@ Moose::Meta::Method::Accessor - A Moose Method metaclass for accessors
 
 =head1 DESCRIPTION
 
-This is a subclass of L<Class::MOP::Method::Accessor> and it's primary
-responsibility is to generate the accessor methods for attributes. It
-can handle both closure based accessors, as well as inlined source based
-accessors.
+This class is a subclass of L<Class::MOP::Method::Accessor> that
+provides additional Moose-specific functionality, all of which is
+private.
 
-This is a fairly new addition to the MOP, but this will play an important
-role in the optimization strategy we are currently following.
-
-=head1 METHODS
-
-=over 4
-
-=item B<generate_accessor_method>
-
-=item B<generate_reader_method>
-
-=item B<generate_writer_method>
-
-=item B<generate_accessor_method_inline>
-
-=item B<generate_reader_method_inline>
-
-=item B<generate_writer_method_inline>
-
-=back
+To understand this class, you should read the the
+L<Class::MOP::Method::Accessor> documentation.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -333,7 +356,7 @@ Yuval Kogman E<lt>nothingmuch@woobling.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
