@@ -4,32 +4,43 @@ use strict;
 use warnings;
 use metaclass;
 
-our $VERSION   = '0.54';
+our $VERSION   = '1.09';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 __PACKAGE__->meta->add_attribute('method_exclusions' => (
-    init_arg => 'excludes',
+    init_arg => '-excludes',
     reader   => 'get_method_exclusions',
     default  => sub { [] }
 ));
 
 __PACKAGE__->meta->add_attribute('method_aliases' => (
-    init_arg => 'alias',
+    init_arg => '-alias',
     reader   => 'get_method_aliases',
     default  => sub { {} }
 ));
 
-sub new { 
+sub new {
     my ($class, %params) = @_;
-    
-    if (exists $params{excludes}) {
-        # I wish we had coercion here :)
-        $params{excludes} = (ref $params{excludes} eq 'ARRAY' 
-                                ? $params{excludes} 
-                                : [ $params{excludes} ]);
+
+    if ( exists $params{excludes} && !exists $params{'-excludes'} ) {
+        $params{'-excludes'} = delete $params{excludes};
     }
-    
-    $class->meta->new_object(%params);
+    if ( exists $params{alias} && !exists $params{'-alias'} ) {
+        $params{'-alias'} = delete $params{alias};
+    }
+
+    if ( exists $params{'-excludes'} ) {
+
+        # I wish we had coercion here :)
+        $params{'-excludes'} = (
+            ref $params{'-excludes'} eq 'ARRAY'
+            ? $params{'-excludes'}
+            : [ $params{'-excludes'} ]
+        );
+    }
+
+    $class->_new(\%params);
 }
 
 sub is_method_excluded {
@@ -57,12 +68,12 @@ sub apply {
     $self->check_role_exclusions(@_);
     $self->check_required_methods(@_);
     $self->check_required_attributes(@_);
-    
+
     $self->apply_attributes(@_);
-    $self->apply_methods(@_);    
-    
+    $self->apply_methods(@_);
+
     $self->apply_override_method_modifiers(@_);
-    
+
     $self->apply_before_method_modifiers(@_);
     $self->apply_around_method_modifiers(@_);
     $self->apply_after_method_modifiers(@_);
@@ -95,6 +106,9 @@ Moose::Meta::Role::Application - A base class for role application
 
 This is the abstract base class for role applications.
 
+The API for this class and its subclasses still needs some
+consideration, and is intentionally not yet documented.
+
 =head2 METHODS
 
 =over 4
@@ -139,9 +153,7 @@ This is the abstract base class for role applications.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -149,7 +161,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
