@@ -4,84 +4,133 @@ use strict;
 use warnings;
 use metaclass;
 
-use Carp            'confess';
-use Scalar::Util    'blessed';
+use Moose::Util  'english_list';
+use Scalar::Util 'weaken', 'blessed';
 
-use Data::Dumper;
-
-our $VERSION   = '0.54';
+our $VERSION   = '1.09';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Moose::Meta::Role::Application';
 
+__PACKAGE__->meta->add_attribute('role' => (
+    reader => 'role',
+));
+
+__PACKAGE__->meta->add_attribute('class' => (
+    reader => 'class',
+));
+
 sub apply {
-    my ($self, $role, $class) = @_;    
+    my ($self, $role, $class) = @_;
+
+    # We need weak_ref in CMOP :(
+    weaken($self->{role}  = $role);
+    weaken($self->{class} = $class);
+
     $self->SUPER::apply($role, $class);
-    $class->add_role($role);        
+
+    $class->add_role($role);
+    $class->add_role_application($self);
 }
 
 sub check_role_exclusions {
     my ($self, $role, $class) = @_;
     if ($class->excludes_role($role->name)) {
-        confess "Conflict detected: " . $class->name . " excludes role '" . $role->name . "'";
+        $class->throw_error("Conflict detected: " . $class->name . " excludes role '" . $role->name . "'");
     }
     foreach my $excluded_role_name ($role->get_excluded_roles_list) {
         if ($class->does_role($excluded_role_name)) {
-            confess "The class " . $class->name . " does the excluded role '$excluded_role_name'";
+            $class->throw_error("The class " . $class->name . " does the excluded role '$excluded_role_name'");
         }
     }
 }
 
 sub check_required_methods {
     my ($self, $role, $class) = @_;
+
+    my @missing;
+    my @is_attr;
+
     # NOTE:
     # we might need to move this down below the
     # the attributes so that we can require any
     # attribute accessors. However I am thinking
     # that maybe those are somehow exempt from
     # the require methods stuff.
-    foreach my $required_method_name ($role->get_required_method_list) {
+    foreach my $required_method ($role->get_required_method_list) {
+        my $required_method_name = $required_method->name;
 
         if (!$class->find_method_by_name($required_method_name)) {
-            
+
             next if $self->is_aliased_method($required_method_name);
-            
-            confess "'" . $role->name . "' requires the method '$required_method_name' " .
-                    "to be implemented by '" . $class->name . "'";
-        }
-        else {
-            # NOTE:
-            # we need to make sure that the method is
-            # not a method modifier, because those do
-            # not satisfy the requirements ...
-            my $method = $class->find_method_by_name($required_method_name);
 
-            # check if it is a generated accessor ...
-            (!$method->isa('Class::MOP::Method::Accessor'))
-                || confess "'" . $role->name . "' requires the method '$required_method_name' " .
-                           "to be implemented by '" . $class->name . "', the method is only an attribute accessor";
-
-            # NOTE:
-            # All other tests here have been removed, they were tests
-            # for overriden methods and before/after/around modifiers.
-            # But we realized that for classes any overriden or modified
-            # methods would be backed by a real method of that name
-            # (and therefore meet the requirement). And for roles, the
-            # overriden and modified methods are "in statis" and so would
-            # not show up in this test anyway (and as a side-effect they
-            # would not fufill the requirement, which is exactly what we
-            # want them to do anyway).
-            # - SL
+            push @missing, $required_method;
         }
     }
+
+    return unless @missing;
+
+    my $error = '';
+
+    @missing = sort { $a->name cmp $b->name } @missing;
+    my @conflicts = grep { $_->isa('Moose::Meta::Role::Method::Conflicting') } @missing;
+
+    if (@conflicts) {
+        my $conflict = $conflicts[0];
+        my $roles = $conflict->roles_as_english_list;
+
+        my @same_role_conflicts = grep { $_->roles_as_english_list eq $roles } @conflicts;
+
+        if (@same_role_conflicts == 1) {
+            $error
+                .= "Due to a method name conflict in roles "
+                .  $roles
+                . ", the method '"
+                . $conflict->name
+                . "' must be implemented or excluded by '"
+                . $class->name
+                . q{'};
+        }
+        else {
+            my $methods
+                = Moose::Util::english_list( map { q{'} . $_->name . q{'} } @same_role_conflicts );
+
+            $error
+                .= "Due to method name conflicts in roles "
+                .  $roles
+                . ", the methods "
+                . $methods
+                . " must be implemented or excluded by '"
+                . $class->name
+                . q{'};
+        }
+    }
+    elsif (@missing) {
+        my $noun = @missing == 1 ? 'method' : 'methods';
+
+        my $list
+            = Moose::Util::english_list( map { q{'} . $_ . q{'} } @missing );
+
+        $error
+            .= q{'}
+            . $role->name
+            . "' requires the $noun $list "
+            . "to be implemented by '"
+            . $class->name . q{'};
+    }
+
+    $class->throw_error($error);
 }
 
 sub check_required_attributes {
-    
+
 }
 
 sub apply_attributes {
     my ($self, $role, $class) = @_;
+    my $attr_metaclass = $class->attribute_metaclass;
+
     foreach my $attribute_name ($role->get_attribute_list) {
         # it if it has one already
         if ($class->has_attribute($attribute_name) &&
@@ -91,8 +140,7 @@ sub apply_attributes {
         }
         else {
             $class->add_attribute(
-                $attribute_name,
-                $role->get_attribute($attribute_name)
+                $role->get_attribute($attribute_name)->attribute_for_class($attr_metaclass)
             );
         }
     }
@@ -101,41 +149,42 @@ sub apply_attributes {
 sub apply_methods {
     my ($self, $role, $class) = @_;
     foreach my $method_name ($role->get_method_list) {
-        
-        next if $self->is_method_excluded($method_name);
-        
-        # it if it has one already
-        if ($class->has_method($method_name) &&
-            # and if they are not the same thing ...
-            $class->get_method($method_name)->body != $role->get_method($method_name)->body) {
-            next;
+        next if $method_name eq 'meta';
+
+        unless ($self->is_method_excluded($method_name)) {
+            # it if it has one already
+            if ($class->has_method($method_name) &&
+                # and if they are not the same thing ...
+                $class->get_method($method_name)->body != $role->get_method($method_name)->body) {
+                next;
+            }
+            else {
+                # add it, although it could be overridden
+                $class->add_method(
+                    $method_name,
+                    $role->get_method($method_name)
+                );
+            }
         }
-        else {
-            # add it, although it could be overriden
-            $class->alias_method(
-                $method_name,
-                $role->get_method($method_name)
-            );         
-        }
-        
+
         if ($self->is_method_aliased($method_name)) {
             my $aliased_method_name = $self->get_method_aliases->{$method_name};
             # it if it has one already
             if ($class->has_method($aliased_method_name) &&
                 # and if they are not the same thing ...
                 $class->get_method($aliased_method_name)->body != $role->get_method($method_name)->body) {
-                confess "Cannot create a method alias if a local method of the same name exists";
-            }            
-            $class->alias_method(
+                $class->throw_error("Cannot create a method alias if a local method of the same name exists");
+            }
+            $class->add_method(
                 $aliased_method_name,
                 $role->get_method($method_name)
-            );                
-        }        
+            );
+        }
     }
     # we must reset the cache here since
     # we are just aliasing methods, otherwise
     # the modifiers go wonky.
-    $class->reset_package_cache_flag;        
+    $class->reset_package_cache_flag;
 }
 
 sub apply_override_method_modifiers {
@@ -210,9 +259,7 @@ Moose::Meta::Role::Application::ToClass - Compose a role into a class
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -220,7 +267,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
