@@ -4,9 +4,11 @@ package Moose::Meta::Role::Method::Conflicting;
 use strict;
 use warnings;
 
+use Moose::Util;
+
 use base qw(Moose::Meta::Role::Method::Required);
 
-our $VERSION   = '0.82';
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -15,6 +17,11 @@ __PACKAGE__->meta->add_attribute('roles' => (
     required => 1,
 ));
 
+sub roles_as_english_list {
+    my $self = shift;
+    Moose::Util::english_list( map { q{'} . $_ . q{'} } @{ $self->roles } );
+}
+
 1;
 
 __END__
@@ -61,13 +68,15 @@ Returns the conflicting method's name, as provided to the constructor.
 Returns the roles that generated this conflicting method, as provided to the
 constructor.
 
+=item B<< $method->roles_as_english_list >>
+
+Returns the roles that generated this conflicting method as an English list.
+
 =back
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -75,7 +84,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2009 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
