@@ -7,7 +7,9 @@ use metaclass;
 
 use Moose::Meta::TypeCoercion::Union;
 
-our $VERSION   = '0.94';
+use List::Util qw(first);
+
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -93,6 +95,12 @@ sub validate {
     return ($message . ' in (' . $self->name . ')') ;
 }
 
+sub find_type_for {
+    my ($self, $value) = @_;
+
+    return first { $_->check($value) } @{ $self->type_constraints };
+}
+
 sub is_a_type_of {
     my ($self, $type_name) = @_;
     foreach my $type (@{$self->type_constraints}) {
@@ -197,6 +205,12 @@ messages returned by the member type constraints.
 A type is considered equal if it is also a union type, and the two
 unions have the same member types.
 
+=item B<< $constraint->find_type_for($value) >>
+
+This returns the first member type constraint for which C<check($value)> is
+true, allowing you to determine which of the Union's member type constraints
+a given value matches.
+
 =item B<< $constraint->is_a_type_of($type_name_or_object) >>
 
 This returns true if any of the member type constraints return true
