@@ -12,21 +12,17 @@ use Try::Tiny ();
 use if ( not our $__mx_is_compiled ), 'Moose::Meta::Class';
 use if ( not our $__mx_is_compiled ), metaclass => 'Moose::Meta::Class';
 
-our $VERSION   = '0.94';
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 sub new {
     my $class = shift;
-
-    my $params = $class->BUILDARGS(@_);
-
     my $real_class = Scalar::Util::blessed($class) || $class;
-    my $self = Class::MOP::Class->initialize($real_class)->new_object($params);
 
-    $self->BUILDALL($params);
+    my $params = $real_class->BUILDARGS(@_);
 
-    return $self;
+    return Class::MOP::Class->initialize($real_class)->new_object($params);
 }
 
 sub BUILDARGS {
@@ -119,11 +115,7 @@ sub does {
     my $meta = Class::MOP::class_of($self);
     (defined $role_name)
         || $meta->throw_error("You must supply a role name to does()");
-    foreach my $class ($meta->class_precedence_list) {
-        my $m = $meta->initialize($class);
-        return 1
-            if $m->can('does_role') && $m->does_role($role_name);
-    }
+    return 1 if $meta->can('does_role') && $meta->does_role($role_name);
     return 0;
 }
 
