@@ -8,7 +8,7 @@ use Sub::Exporter;
 use Scalar::Util 'blessed';
 use Class::MOP   0.60;
 
-our $VERSION   = '0.82';
+our $VERSION   = '1.09';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -18,12 +18,15 @@ my @exports = qw[
     search_class_by_role
     ensure_all_roles
     apply_all_roles
+    with_traits
     get_all_init_args
     get_all_attribute_values
     resolve_metatrait_alias
     resolve_metaclass_alias
     add_method_modifier
     english_list
+    meta_attribute_alias
+    meta_class_alias
 ];
 
 Sub::Exporter::setup_exporter({
@@ -49,12 +52,14 @@ sub does_role {
 }
 
 sub search_class_by_role {
-    my ($class_or_obj, $role_name) = @_;
+    my ($class_or_obj, $role) = @_;
 
     my $meta = find_meta($class_or_obj);
 
     return unless defined $meta;
 
+    my $role_name = blessed $role ? $role->name : $role;
+
     foreach my $class ($meta->class_precedence_list) {
 
         my $_meta = find_meta($class);
@@ -79,7 +84,7 @@ sub ensure_all_roles {
 
 sub apply_all_roles {
     my $applicant = shift;
-    _apply_all_roles($applicant, sub { 1 }, @_);
+    _apply_all_roles($applicant, undef, @_);
 }
 
 sub _apply_all_roles {
@@ -93,9 +98,17 @@ sub _apply_all_roles {
 
     my $roles = Data::OptList::mkopt( [@_] );
 
+    my @role_metas;
     foreach my $role (@$roles) {
-        Class::MOP::load_class( $role->[0] );
-        my $meta = Class::MOP::class_of( $role->[0] );
+        my $meta;
+
+        if ( blessed $role->[0] ) {
+            $meta = $role->[0];
+        }
+        else {
+            Class::MOP::load_class( $role->[0] , $role->[1] );
+            $meta = Class::MOP::class_of( $role->[0] );
+        }
 
         unless ($meta && $meta->isa('Moose::Meta::Role') ) {
             require Moose;
@@ -103,24 +116,37 @@ sub _apply_all_roles {
                     . $role->[0]
                     . " is not a Moose role" );
         }
+
+        push @role_metas, [ $meta, $role->[1] ];
     }
 
-    @$roles = grep { local $_ = $_->[0]; $role_filter->() } @$roles;
+    if ( defined $role_filter ) {
+        @role_metas = grep { local $_ = $_->[0]; $role_filter->() } @role_metas;
+    }
 
-    return unless @$roles;
+    return unless @role_metas;
 
     my $meta = ( blessed $applicant ? $applicant : find_meta($applicant) );
 
-    if ( scalar @$roles == 1 ) {
-        my ( $role, $params ) = @{ $roles->[0] };
-        my $role_meta = Class::MOP::class_of($role);
-        $role_meta->apply( $meta, ( defined $params ? %$params : () ) );
+    if ( scalar @role_metas == 1 ) {
+        my ( $role, $params ) = @{ $role_metas[0] };
+        $role->apply( $meta, ( defined $params ? %$params : () ) );
     }
     else {
-        Moose::Meta::Role->combine( @$roles )->apply($meta);
+        Moose::Meta::Role->combine(@role_metas)->apply($meta);
     }
 }
 
+sub with_traits {
+    my ($class, @roles) = @_;
+    return $class unless @roles;
+    return Moose::Meta::Class->create_anon_class(
+        superclasses => [$class],
+        roles        => \@roles,
+        cache        => 1,
+    )->name;
+}
+
 # instance deconstruction ...
 
 sub get_all_attribute_values {
@@ -146,6 +172,15 @@ sub resolve_metatrait_alias {
     return resolve_metaclass_alias( @_, trait => 1 );
 }
 
+sub _build_alias_package_name {
+    my ($type, $name, $trait) = @_;
+    return 'Moose::Meta::'
+         . $type
+         . '::Custom::'
+         . ( $trait ? 'Trait::' : '' )
+         . $name;
+}
+
 {
     my %cache;
 
@@ -156,12 +191,9 @@ sub resolve_metatrait_alias {
         return $cache{$cache_key}{$metaclass_name}
             if $cache{$cache_key}{$metaclass_name};
 
-        my $possible_full_name
-            = 'Moose::Meta::'
-            . $type
-            . '::Custom::'
-            . ( $options{trait} ? "Trait::" : "" )
-            . $metaclass_name;
+        my $possible_full_name = _build_alias_package_name(
+            $type, $metaclass_name, $options{trait}
+        );
 
         my $loaded_class = Class::MOP::load_first_existing_class(
             $possible_full_name,
@@ -177,7 +209,10 @@ sub resolve_metatrait_alias {
 
 sub add_method_modifier {
     my ( $class_or_obj, $modifier_name, $args ) = @_;
-    my $meta                = find_meta($class_or_obj);
+    my $meta
+        = $class_or_obj->can('add_before_method_modifier')
+        ? $class_or_obj
+        : find_meta($class_or_obj);
     my $code                = pop @{$args};
     my $add_modifier_method = 'add_' . $modifier_name . '_method_modifier';
     if ( my $method_modifier_type = ref( @{$args}[0] ) ) {
@@ -188,6 +223,18 @@ sub add_method_modifier {
             $meta->$add_modifier_method( $_->name, $code )
                 for @matched_methods;
         }
+        elsif ($method_modifier_type eq 'ARRAY') {
+            $meta->$add_modifier_method( $_, $code ) for @{$args->[0]};
+        }
+        else {
+            $meta->throw_error(
+                sprintf(
+                    "Methods passed to %s must be provided as a list, arrayref or regex, not %s",
+                    $modifier_name,
+                    $method_modifier_type,
+                )
+            );
+        }
     }
     else {
         $meta->$add_modifier_method( $_, $code ) for @{$args};
@@ -207,6 +254,37 @@ sub english_list {
     return $list;
 }
 
+sub _caller_info {
+    my $level = @_ ? ($_[0] + 1) : 2;
+    my %info;
+    @info{qw(package file line)} = caller($level);
+    return \%info;
+}
+
+sub _create_alias {
+    my ($type, $name, $trait, $for) = @_;
+    my $package = _build_alias_package_name($type, $name, $trait);
+    Class::MOP::Class->initialize($package)->add_method(
+        register_implementation => sub { $for }
+    );
+}
+
+sub meta_attribute_alias {
+    my ($to, $from) = @_;
+    $from ||= caller;
+    my $meta = Class::MOP::class_of($from);
+    my $trait = $meta->isa('Moose::Meta::Role');
+    _create_alias('Attribute', $to, $trait, $from);
+}
+
+sub meta_class_alias {
+    my ($to, $from) = @_;
+    $from ||= caller;
+    my $meta = Class::MOP::class_of($from);
+    my $trait = $meta->isa('Moose::Meta::Role');
+    _create_alias('Class', $to, $trait, $from);
+}
+
 1;
 
 __END__
@@ -246,16 +324,19 @@ This method takes a class name or object and attempts to find a
 metaclass for the class, if one exists. It will B<not> create one if it
 does not yet exist.
 
-=item B<does_role($class_or_obj, $role_name)>
+=item B<does_role($class_or_obj, $role_or_obj)>
 
-Returns true if C<$class_or_obj> does the given C<$role_name>.
+Returns true if C<$class_or_obj> does the given C<$role_or_obj>. The role can
+be provided as a name or a L<Moose::Meta::Role> object.
 
-The class must already have a metaclass for this to work.
+The class must already have a metaclass for this to work. If it doesn't, this
+function simply returns false.
 
-=item B<search_class_by_role($class_or_obj, $role_name)>
+=item B<search_class_by_role($class_or_obj, $role_or_obj)>
 
 Returns the first class in the class's precedence list that does
-C<$role_name>, if any.
+C<$role_or_obj>, if any. The role can be either a name or a
+L<Moose::Meta::Role> object.
 
 The class must already have a metaclass for this to work.
 
@@ -266,15 +347,20 @@ applicant can be a role name, class name, or object.
 
 The C<$applicant> must already have a metaclass object.
 
-The list of C<@roles> should be a list of names, each of which can be
-followed by an optional hash reference of options (C<exclude> and
-C<alias>).
+The list of C<@roles> should a list of names or L<Moose::Meta::Role> objects,
+each of which can be followed by an optional hash reference of options
+(C<-excludes> and C<-alias>).
 
 =item B<ensure_all_roles($applicant, @roles)>
 
 This function is similar to L</apply_all_roles>, but only applies roles that
 C<$applicant> does not already consume.
 
+=item B<with_traits($class_name, @role_names)>
+
+This function creates a new class from C<$class_name> with each of
+C<@role_names> applied. It returns the name of the new class.
+
 =item B<get_all_attribute_values($meta, $instance)>
 
 Returns a hash reference containing all of the C<$instance>'s
@@ -300,8 +386,16 @@ when specifying the C<metaclass> or C<traits> option for an attribute:
         metaclass => "Bar",
     );
 
-The name resolution mechanism is covered in L<Moose/Trait Name
-Resolution>.
+The name resolution mechanism is covered in
+L<Moose/Metaclass and Trait Name Resolution>.
+
+=item B<meta_class_alias($to[, $from])>
+
+=item B<meta_attribute_alias($to[, $from])>
+
+Create an alias from the class C<$from> (or the current package, if
+C<$from> is unspecified), so that
+L<Moose/Metaclass and Trait Name Resolution> works properly.
 
 =item B<english_list(@items)>
 
@@ -325,9 +419,7 @@ Here is a list of possible functions to write
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
